"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.stopServer = exports.startServer = void 0;
const tslib_1 = require("tslib");
const fs_1 = tslib_1.__importDefault(require("fs"));
const kill_port_1 = tslib_1.__importDefault(require("kill-port"));
const koa_1 = tslib_1.__importDefault(require("koa"));
const open_1 = tslib_1.__importDefault(require("open"));
const client_1 = require("@/client");
const socket_server_1 = require("@/socket-server");
const log_1 = require("@/utils/log");
const db_1 = require("@/db");
const router_1 = require("@/router");
const config_1 = require("@/config");
const log = new log_1.Logger('application');
let server;
let socketServer;
/**
 * 开启调试服务
 */
const startServer = () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    log.info('start server argv: %j', global.appArgv);
    yield init();
    const { host, port, useAdb, useIWDP, useTunnel, open: openChrome, isRemote } = global.appArgv;
    const app = new koa_1.default();
    router_1.routeApp(app);
    server = app.listen(port, host, () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        var _a;
        log.info('start debug server success.');
        if (!isRemote) {
            const { startTunnel, startIWDP, startAdbProxy } = yield Promise.resolve().then(() => tslib_1.__importStar(require('./child-process/index')));
            if (useTunnel)
                startTunnel();
            else if (useIWDP)
                startIWDP();
            if (useAdb)
                startAdbProxy();
            if (openChrome) {
                const url = `http://${host}:${port}/extensions/home.html`;
                try {
                    open_1.default(url, { app: { name: open_1.default.apps.chrome } });
                }
                catch (e) {
                    log.error('open %s by chrome failed, please open manually, %s', url, (_a = e) === null || _a === void 0 ? void 0 : _a.stack);
                }
            }
        }
        socketServer = new socket_server_1.SocketServer(server);
        socketServer.start();
    }));
    server.on('close', () => {
        log.warn('debug server is closed.');
    });
});
exports.startServer = startServer;
/**
 * 停止调试服务
 */
const stopServer = (exitProcess = false, ...arg) => {
    var _a;
    try {
        log.info('stopServer %j', arg);
        if (server) {
            server.close();
            server = null;
        }
        if (exitProcess)
            process.exit(0);
    }
    catch (e) {
        log.error('stopServer error, %s', (_a = e) === null || _a === void 0 ? void 0 : _a.stack);
    }
};
exports.stopServer = stopServer;
/**
 * 服务初始化
 */
const init = () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    var _b, _c;
    const { cachePath } = config_1.config;
    try {
        fs_1.default.rmdirSync(cachePath, { recursive: true });
    }
    catch (e) {
        log.warn('rm cache dir error: %s', (_b = e) === null || _b === void 0 ? void 0 : _b.stack);
    }
    yield fs_1.default.promises.mkdir(cachePath, { recursive: true });
    yield db_1.initDbModel();
    client_1.initAppClient();
    const { port, iWDPPort, clearAddrInUse } = global.appArgv;
    if (clearAddrInUse) {
        try {
            yield kill_port_1.default(port, 'tcp');
            yield kill_port_1.default(iWDPPort, 'tcp');
        }
        catch (e) {
            log.error('Address %s %s already in use! %s', port, iWDPPort, (_c = e) === null || _c === void 0 ? void 0 : _c.stack);
            return process.exit(1);
        }
    }
});
